// stud_arr.cpp
// 5/4/99
// Demo of simple array operations with records as elements
// Ken Brownsey


#include <iostream.h>
#include "my_const.h"
#include "stu_rec.h"


#define Max 5


typedef StudentRec_T StudentArray_T[Max];
typedef struct {
	StudentArray_T	Students;
	int			NextVacancy;
} StudentTable_T;

StudentTable_T	Table;


//	PRE Q_Full(Tab) is FALSE
// POST NewStud is added to Tab
void
Add_Stud(StudentRec_T NewStud, StudentTable_T &Tab) {
	Tab.Students[Tab.NextVacancy] = NewStud;
	Tab.NextVacancy++;
}


// PRE TRUE
// POST 	IF Tab is full RETURNS TRUE
//       ELSE RETURNS FALSE
BOOLEAN
Q_Full(StudentTable_T Tab) {
	return(Tab.NextVacancy == Max);
}


// PRE TRUE
// POST 	IF Tab is empty RETURNS TRUE
//       ELSE RETURNS FALSE
BOOLEAN
Q_Empty(StudentTable_T Tab) {
	return(Tab.NextVacancy == 0);
}

// PRE	TRUE
// POST	IF Stud is in Table RETURNS TRUE
//			ELSE RETURNS FALSE
BOOLEAN
Q_Present(Key_T Key, StudentTable_T Table) {
	int Index;
	BOOLEAN Q_Found;

	Q_Found = FALSE;

	for (Index =0; (Index < Table.NextVacancy) && !Q_Found; Index++) {
		if (Get_Key(Table.Students[Index]) == Key) {
			Q_Found = TRUE;
		}
	}
	return(Q_Found);
}


// PRE	Student record with key value Key is in Tab
// POST  Student record is removed from Tab
void
Delete_Stud(Key_T Key, StudentTable_T &Table) {
	int	Index;
	BOOLEAN Q_Found;

	Q_Found = FALSE;

	for (Index =0; !Q_Found; Index++) {
		if (Get_Key(Table.Students[Index]) == Key) {
			Q_Found = TRUE;
		}
	}
	/* Index is now one past where we want to delete so..*/
	Index--;

	Table.NextVacancy--;

	for (; Index < Table.NextVacancy; Index++) {
		Table.Students[Index] = Table.Students[Index + 1];
	}
}


// PRE TRUE
// POST	IF Q_Full(Tab) appropriate message is printed
//			ELSE user is prompted for number to be added to Tab
void
Read_NewStud(StudentTable_T &Tab) {
	StudentRec_T	NewStud;

	if (Q_Full(Tab)) {
		cout 	<< "Sorry - table is now full\n";
		cout 	<< "You need to delete some to put new records in \n";
	}
	else {
		cout	<< "Please type in a new record to be added ==> ";
		Read_Rec(NewStud);
		Add_Stud(NewStud, Tab);
	}
}


// PRE TRUE
// POST 	prompts user for value
//			IF value is in Tab prints out message to say so
//       ELSE prints out message to show it is not there
void
Check_Stud(StudentTable_T Tab) {
	Key_T	Key;

	if (!Q_Empty(Tab)){
		cout 	<< "Please type in key of student record to check ==> ";
		Key = Read_Key();

		if (Q_Present(Key, Tab)) {
			cout	<< "Student record with key ";
			Write_Key(Key);
			cout	<< " is in the table\n";
		}
		else {
			cout	<< "There is no student record with key ";
			Write_Key(Key);
			cout	<< " in the table\n";
		}
	}
	else {
		cout	<< ("\nTable is empty - nothing to check\n");
	}
}


// PRE	TRUE
// POST  prompts user for value
//			IF value is in table it is removed and
//				message to show success
//			ELSE message to say that it is not present
void
Remove_Stud(StudentTable_T &Tab) {

	Key_T	Key;

	if (!Q_Empty(Tab)){
		cout 	<< "Please type in the key value of a student record to delete ==> ";
		Key = Read_Key();

		if (Q_Present(Key, Tab)) {
			Delete_Stud(Key, Table);
			cout	<< "Student record with key ";
			Write_Key(Key);
			cout	<< " has been deleted from the table\n";
		}
		else {
			cout	<< "Student record with key ";
			Write_Key(Key);
			cout	<< " is not in the table to begin with\n";
		}
	}
	else {
		cout	<< ("\nTable is empty - nothing to delete\n");
	}
}


//	PRE	TRUE
//	POST	IF Tab is not empty all values are printed to screen
//			ELSE message to say that Tab is empty
void
Print_All(StudentTable_T Tab) {

	int	Index;

	if (!Q_Empty(Tab)) {
		cout	<< "\n******************\nContents of the table are as follows\n\n";
		for (Index = 0; Index < Tab.NextVacancy; Index++) {
			cout	<< "Record in position "
					<< (Index + 1)
					<< " is as follows ";
			Write_Rec(Tab.Students[Index]);
			cout	<< "\n";
		}
		cout	<< ("\n******************\nThat's all!\n");
	}
	else {
		cout	<< ("\nTable is empty - nothing to print\n\n");
	}
}



//	PRE TRUE
//	POST menu is displayed
void
Menu() {
	cout 	<< "show (m)enu\n";
	cout 	<< "(r)ead in new value\n";
	cout 	<< "(c)heck value is in table\n";
	cout 	<< "r(e)move value from table\n";
	cout 	<< "(p)rint all values in table\n";
	cout  << "e(x)it\n";
}


void
Initialise(StudentTable_T &Tab) {
	cout	<< "*********************************\n";
	cout	<< "The great student storage program\n";
	cout	<< "*********************************";
	Tab.NextVacancy = 0;
}


void
Loop(StudentTable_T &Tab) {
	char	Option;


	for(Option = 'm'; Option != 'x';) {
		switch (Option) {
			case 'm':
				Menu();
			break;
			case 'r':
				Read_NewStud(Tab);
			break;
			case 'c':
				Check_Stud(Tab);
			break;
			case 'e':
				Remove_Stud(Tab);
			break;
			case 'p':
				Print_All(Tab);
			break;
			default:
				Menu();
			break;
		}
		cout 	<< "\nPlease type next option ==> ";
		cin   >> Option;
	}
}


void
Finalise(StudentTable_T Tab) {
	cout	<< "*********************************\n";
	cout	<< "That's it folks..................\n";
	cout	<< "*********************************";
}


void
main() {
	Initialise(Table);
	Loop(Table);
	Finalise(Table);
}



