// hashdemo.cpp
// 6/7/99
// Demo of hashing
// Ken Brownsey

#include <iostream.h>
#include "hashtab.h"
#include <stdlib.h>

//	PRE	TRUE
//	POST	HashTable is emptied
//			AND HashTable' contains NumLocations locations
//			containing Size values, randomly chosen from
//			range [1..Hi]
void
Make_Random_HashTable(HashTable_T &HashTable, int NumLocations, int Size, int Hi) {
	int	Index;
	int	NextVal;

	if (HashTable != NULL) {
		Delete_HashTable(HashTable);
	}
	Create_HashTable(HashTable, NumLocations);
	for (Index = 1; Index <= Size; Index++) {
		NextVal = rand()%Hi + 1;
		for (;Q_PresentValue(NextVal, HashTable);) {
			NextVal = rand()%Hi + 1;
		}
		Add_Value(NextVal, HashTable);
	}
}


// PRE 	TRUE
// POST 	prompts user for key value
//			IF key value is in HashTable
//				prints out message to say so
//       ELSE prints out
//				message to show it is not there
void
Check_Value(HashTable_T HashTable) {
	int 	Value;

	if (!Q_EmptyHashTable(HashTable)){
		cout 	<< "Please type in value to check ==> ";
		cin	>> Value;

		if (Q_PresentValue(Value, HashTable)) {
			cout	<< "Value  "
					<<	Value
					<< " is in the hash table\n";
		}
		else {
			cout	<< "There is no value "
					<<	Value
					<< " in the hash table\n";
		}
	}
	else {
		cout	<< ("\nHashTable is empty - nothing to check\n");
	}
}


// PRE 	TRUE
// POST 	prompts user for key value
//			IF key value is in HashTable
//				prints out message to say so
//       ELSE
//				adds it to HashTable
void
Add_NewValue(HashTable_T HashTable) {
	int 	Value;

	cout 	<< "Please type in value to add ==> ";
	cin	>> Value;
	if (Q_PresentValue(Value, HashTable)) {
		cout	<< "Value  "
				<<	Value
				<< " is already in the hash table\n";
	}
	else {
		cout	<< "Adding "
				<<	Value
				<< " to the hash table\n";
		Add_Value(Value, HashTable);
	}
}


//	PRE	TRUE
//	POST	IF HashTable is not empty all values are printed to screen
//			ELSE message to say that HashTable is empty
void
Print_All_Values(HashTable_T HashTable) {

	if (!Q_EmptyHashTable(HashTable)) {
		cout	<< "\n******************\nValues of the table are as follows\n\n";
		Print_HashTable(HashTable);
	}
	else {
		cout	<< ("\nDatabase is empty - no keys to print\n\n");
	}
}


//	PRE	TRUE
//	POST	User is prompted for Size of OrdHashTable
//			IF Size > 0
//				OrdHashTable contains Size elements in order
//			ELSE
//				message
void
Build_RandomHashTable(HashTable_T &HashTable) {
	int	Size;
	int	NumLocations;
	int	MaxVal;

	cout	<< "\nPlease type in number of locations ==> ";
	cin	>>	NumLocations;
	if (NumLocations > 0 ) {
		cout	<< "\nPlease type in number of elements ==> ";
		cin	>>	Size;
		if (Size > 0 ) {
			cout	<< "\nPlease type in value of biggest element ==> ";
			cin	>>	MaxVal;
			if (MaxVal > 0 ) {
				if (Size > MaxVal) {
					cout	<< "\nImpossible to allocate "
							<<	Size
							<< " elements in range [1.."
							<<	MaxVal
							<<	"]\n";
				}
				else {
					Make_Random_HashTable(HashTable, NumLocations, Size, MaxVal);
				}
			}
			else {
				cout	<<	"\n"
						<<	MaxVal
						<<	" is less than 0";
			}
		}
		else {
			cout	<<	"\n"
					<<	Size
					<<	" is less than 0";
		}
	}
	else {
		cout	<<	"\n"
				<<	NumLocations
				<<	" is less than 0";
	}
}



// PRE  -	TRUE
// POST -	menu displayed
void
Menu() {
	cout	<<	"***	(a)dd new record to hash table ***********\n"
			<< "***	(p)rint all values        ****************\n"
			<< "***	build (r)andom hash table             ****\n"
			<< "***	(c)heck value in hash table           ****\n"
			<< "***	make database (e)mpty  *******************\n"
			<< "***   e(x)it					      ***************\n"
			<< "***   (m)enu						   ***************\n";
}


// PRE	TRUE
// POST	Sets up
void
Initialise(HashTable_T &HashTable) {

	Create_HashTable(HashTable, 0);
	cout	<< "Here we go !\n";
}


// PRE	TRUE
// POST Goes into a prompt, read and execute cycle
//		until 'x' is the character read
void
Loop(HashTable_T &HashTable) {

	char	Request;
	int	RequestCount;

	for (Request = 'm', RequestCount = 0; Request != 'x'; RequestCount++) {

		switch(Request) {
			case 'a' :	Add_NewValue(HashTable);
				break;
			case 'p' :	Print_All_Values(HashTable);
				break;
			case 'c' :	Check_Value(HashTable);
				break;
			case 'r' :	Build_RandomHashTable(HashTable);
				break;
			case 'e' :	Delete_HashTable(HashTable);
				break;
			case 'm' : Menu();
				break;
			default   :
				cout	<< "\n m for menu \n";
		}
		cout	<< "\n\nNext request ["
				<< RequestCount
				<< "] ==> ";
		cin	>> Request;
	}
}


// PRE	TRUE
// POST	Clears up
void
Finalise(HashTable_T &HashTable) {
	Delete_HashTable(HashTable);
	cout	<< "That is it !\n";
}


void
main() {
	HashTable_T HashTable;

	Initialise(HashTable);
	Loop(HashTable);
	Finalise(HashTable);
}
