// searchdb.cpp
// 6/7/99
// Linear structure for demo of searching ordered and unordered sequence
// Ken Brownsey


#include <iostream.h>
#include "searchdb.h"
#include <stdlib.h>

//	PRE	Keys in DB are in ascedning order
// POST	DB' = DB with NewRec added
//       AND keys in DB' are in order
void
Add_Value(int Value, SearchDB_T DB) {

	if (Q_Empty(DB)) {
		Append_After_Current(Value, DB);
	}
	else {
		Go_To_Head(DB);

		for (; (!Q_At_End(DB)) && (Value > Get_Current(DB));) {
			Go_To_Next(DB);
		}
		// EITHER 	Q_At_End AND Value > Get_Current(DB)
		// OR			Q_At_End AND Value <= Get_Current(DB)
		// OR			NOT Q_At_End AND Q_At_End AND Value <= Get_Current(DB)
		if (Value <= Get_Current(DB)) {
			Insert_Before_Current(Value, DB);
		}
		else {
			Append_After_Current(Value, DB);
		}
	}
}


// PRE	DB is in ascending key order
// POST	IF Value is in DB
//				Position is position in DB
//				RETURNS TRUE
//			ELSE
//				IF at least one value in DB > Value
//					Position is that of the
//					next largest to Value
//				ELSE
//					Position is that of last in DB
//				RETURNS FALSE
BOOLEAN
Q_PresentOrd(int Value, SearchDB_T DB, int &Position) {

	Go_To_Head(DB);
	for (	Position = 1; (!Q_At_End(DB)) && (Value > Get_Current(DB)); Position++) {
		Go_To_Next(DB);
	}
	// EITHER 	Q_At_End AND Value > Get_Current(DB)
	// OR			Q_At_End AND Value <= Get_Current(DB)
	// OR			NOT Q_At_End AND Q_At_End AND Value <= Get_Current(DB)
	return(Value == Get_Current(DB));
}


// PRE	TRUE
// POST	IF Value is in DB
//				Position is position of Value in DB
//				RETURNS TRUE
//			ELSE
//				Position' is that of last in DB
//				AND	Q_At_End(DB')
//				RETURNS FALSE
BOOLEAN
Q_PresentUnOrd(int Value, SearchDB_T DB, int &Position) {

	Go_To_Head(DB);
	for (Position = 1; (!Q_At_End(DB)) && (Value != Get_Current(DB));Position++) {
		Go_To_Next(DB);
	}
	// EITHER 	Q_At_End AND Value != Get_Current(DB)
	// OR			Q_At_End AND Value == Get_Current(DB)
	// OR			NOT Q_At_End AND Q_At_End AND Value == Get_Current(DB)
	return(Value == Get_Current(DB));
}


//	PRE	Size > 0
// POST	OrdRandDB contains Size distinct
//			integers in the range 1..2 * Size
//			in ascending order
void
Make_Ordered_RandomDB(SearchDB_T OrdRandDB, int Size) {
	int	Index;
	int	NextVal;
	int	Dummy;

	for(Index = 1; Index <= Size; Index++) {
		NextVal = 1 + rand() % (2 * Size);
		for(;Q_PresentOrd(NextVal, OrdRandDB, Dummy);) {
			// try again if latest NextVal is already in
			NextVal = 1 + rand() % (2 * Size);
		}
		Add_Value(NextVal, OrdRandDB);
	}
}



//	PRE	Size > 0
// POST	UnOrdRandDB contains Size distinct
//			integers in the range 1..2 * Size
//			in random order
void
Make_UnOrdered_RandomDB(SearchDB_T UnOrdRandDB, int Size) {
	int	Index;
	int	NextVal;
	int	Dummy;

	Append_After_Current(1 + rand() % (2 * Size), UnOrdRandDB);

	for(Index = 2; Index <= Size; Index++) {
		NextVal = 1 + rand() % (2 * Size);
		for(;Q_PresentUnOrd(NextVal, UnOrdRandDB, Dummy);) {
			// try again if latest NextVal is already in
			NextVal = 1 + rand() % (2 * Size);
		}
		Append_After_Current(NextVal, UnOrdRandDB);
		Go_To_Next(UnOrdRandDB);
	}
}


//	PRE	TRUE
//	POST 	Q_Empty(DB)
void
Make_Empty(SearchDB_T DB) {

	for(;!Q_Empty(DB);) {
		Delete_Current(DB);
	}
}




