// Ken Brownsey
// 14/5/99
// arc_bst.cpp
// ArcBST_T directly pointer based implementation

#include "stack.h"
#include "my_const.h"
#include "arc_bst.h"
#include "vert_bst.h"


// record structure for iterator
typedef struct {
		Stack_T Stack;
		SElement_T Current;
		int CurrentNumber;
		BOOLEAN	Q_Bound;
}	ArcBST_ItRec_T, *ArcBST_ItRec_Ptr_T;


typedef struct TN_T {
	struct TN_T	*Left_Ptr,
					*Right_Ptr;
	ArcKAPRec_T	Element;
} ArcNode_T, *ArcNode_Ptr_T;




// PRE	TRUE
// POST	IF ArcBST contains records with key Key
//				RETURNS tree containing first occurrence as root
//			ELSE
//				RETURNS the empty tree
ArcBST_T
Find_First_R(int (*Get_Key) (ArcKAPRec_T), int Key, ArcBST_T ArcBST) {
	ArcNode_Ptr_T	Arc_Ptr;

	if (ArcBST == NULL) {
		return(ArcBST);
	}
	else {
		Arc_Ptr = (ArcNode_Ptr_T) ArcBST;
		if (Key < Get_Key(Arc_Ptr->Element)) {
			return(Find_First_R(Get_Key, Key, (void *) Arc_Ptr->Left_Ptr));
		}
		else {
			if (Key > Get_Key(Arc_Get_Root(ArcBST))) {
				return(Find_First_R(Get_Key, Key, (void *) Arc_Ptr->Right_Ptr));
			}
			else {
				return(ArcBST);
			}
		}
	}
}


// PRE	TRUE
// POST	IF record with from key Key
//			is in ArcBST apart from the root
//				(different from Find_First_R)
//				RETURNS tree with that record as root
//			ELSE
//				RETURNS the empty tree
ArcBST_T
Find_NextDuplicate(int (*Get_Key) (ArcKAPRec_T), int Key, ArcBST_T ArcBST) {
	ArcNode_Ptr_T	Arc_Ptr;

	Arc_Ptr = (ArcNode_Ptr_T) ArcBST;
	return(Find_First_R(Get_Key, Key, (void *) Arc_Ptr->Right_Ptr));
}


// PRE	TRUE
// POST	RETURNS the empty tree
ArcBST_T
Create_ArcTree() {
	return(NULL);
}


// PRE	TRUE
// POST	if (Tree is empty RETURNS TRUE else RETURNS FALSE
BOOLEAN
Q_EmptyArcTree(ArcBST_T Tree) {
	return(Tree == NULL);
}


// PRE  	Tree is not empty
// POST	RETURNS the root record
ArcKAPRec_T
Arc_Get_Root(ArcBST_T Tree) {
	ArcNode_Ptr_T	Arc_Ptr;

	Arc_Ptr = (ArcNode_Ptr_T) Tree;

	return(Arc_Ptr->Element);
}


// PRE	TRUE
// POST	if (Tree is not empty
//				RETURNS the left sub tree of Tree
//			else
//				RETURNS the empty tree
ArcBST_T
Arc_Get_Left(ArcBST_T Tree) {
	ArcNode_Ptr_T	Arc_Ptr;

	Arc_Ptr = (ArcNode_Ptr_T) Tree;

	if (Arc_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((ArcBST_T) (Arc_Ptr->Left_Ptr));
	}
}


// PRE	TRUE
// POST	if (Tree is not empty
//				RETURNS the right sub tree of Tree
//			else
//				RETURNS the empty tree
ArcBST_T
ArcGet_Right(ArcBST_T Tree) {
	ArcNode_Ptr_T	Arc_Ptr;

	Arc_Ptr = (ArcNode_Ptr_T) Tree;

	if (Arc_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((ArcBST_T)  Arc_Ptr->Right_Ptr);
	}
}


// PRE	TRUE
// POST	RETURNS the tree whose root is Root,
//				left sub tree is Left,
//				right sub tree is Right
ArcBST_T
Make_ArcTree(ArcBST_T Left, ArcKAPRec_T Element, ArcBST_T Right) {

	ArcNode_Ptr_T	NewTree;

	NewTree = new ArcNode_T;
	NewTree->Element = Element;
	NewTree->Left_Ptr = (ArcNode_Ptr_T) Left;
	NewTree->Right_Ptr = (ArcNode_Ptr_T) Right;
	return((ArcBST_T)  NewTree);
}


// PRE	TRUE
// POST	if record with key ArcKey is in ArcBST
//				RETURNS TRUE
//			else
//				RETURNS FALSE
BOOLEAN
Q_ArcBST_Present_R(int (*Get_Key) (ArcKAPRec_T), int ArcKey, ArcBST_T ArcBST) {
	ArcNode_Ptr_T	Arc_Ptr;

	Arc_Ptr = (ArcNode_Ptr_T) ArcBST;

	if (ArcBST == NULL) {
		return(FALSE);
	}
	else {
		if (ArcKey < Get_Key(Arc_Ptr->Element)) {
			return(Q_ArcBST_Present_R(Get_Key, ArcKey, Arc_Ptr->Left_Ptr));
		}
		else {
			if (ArcKey > Get_Key(Arc_Get_Root(ArcBST))) {
				return(Q_ArcBST_Present_R(Get_Key, ArcKey, Arc_Ptr->Right_Ptr));
			}
			else {
				return(TRUE);
			}
		}
	}
}


// PRE	NOT Q_ArcBST_Present_R(Rec, ArcBST)
// POST	RETURNS ArcBST with Rec inserted
void
ArcBST_Insert_R(int (*Get_Key) (ArcKAPRec_T), ArcKAPRec_T Rec, ArcBST_T &ArcBST) {
	int ArcKey;
	ArcNode_Ptr_T	Arc_Ptr,
					NewArc_Ptr,
					Parent;
	BOOLEAN	Q_OKInsert;

	Arc_Ptr = (ArcNode_Ptr_T) ArcBST;
	NewArc_Ptr = new(ArcNode_T);
	NewArc_Ptr->Left_Ptr 	= NULL;
	NewArc_Ptr->Element 	= Rec;
	NewArc_Ptr->Right_Ptr 	= NULL;
	if (Arc_Ptr == NULL) {
		Arc_Ptr = NewArc_Ptr;
	}
	else {
		Parent = Arc_Ptr;
		Q_OKInsert = FALSE;
		ArcKey = Get_Key(Rec);
		for(;!Q_OKInsert;) {
			if (ArcKey < Get_Key(Parent->Element)) {
				if (Parent->Left_Ptr == NULL) {
					Parent->Left_Ptr = NewArc_Ptr;
					Q_OKInsert = TRUE;
				}
				else {
					Parent = Parent->Left_Ptr;
				}
			}
			else {
				if (Parent->Right_Ptr == NULL) {
					Parent->Right_Ptr = NewArc_Ptr;
					Q_OKInsert = TRUE;
				}
				else {
					Parent = Parent->Right_Ptr;
				}
			}
		}
	}
	ArcBST = (ArcBST_T)  Arc_Ptr;
}


// PRE	Q_ArcBST_Present_R(ArcKey, ArcBST)
// POST	RETURNS record with key value ArcKey from ArcBST
ArcKAPRec_T
ArcBST_Retrieve_R(int (*Get_Key) (ArcKAPRec_T), int ArcKey, ArcBST_T ArcBST) {
	ArcNode_Ptr_T	Arc_Ptr;

	Arc_Ptr = (ArcNode_Ptr_T) ArcBST;

	if (ArcKey < Get_Key(Arc_Ptr->Element)) {
		return(ArcBST_Retrieve_R(Get_Key, ArcKey, Arc_Ptr->Left_Ptr));
	}
	else {
		if (ArcKey > Get_Key(Arc_Ptr->Element)) {
			return(ArcBST_Retrieve_R(Get_Key, ArcKey, Arc_Ptr->Right_Ptr));
		}
		else {
			return(Arc_Ptr->Element);
		}
	}
}


// PRE	NOT Q_EmptyTree(ArcBST)
// POST	RETURNS rightmost ArcKAPRec_T of ArcBST
ArcKAPRec_T
Get_LeftMostArc_R(	ArcNode_Ptr_T	Arc_Ptr) {

	if (Arc_Ptr->Left_Ptr == NULL) {
		return(Arc_Ptr->Element);
	}
	else {
		return(Get_LeftMostArc_R(Arc_Ptr->Left_Ptr));
	}
}


// PRE	TRUE
// POST	if (Q_ArcBST_Present(Rec, ArcBST)
//				RETURNS ArcBST with Rec deleted
//			else
//				RETURNS ArcBST
void
ArcBST_Delete_R(int (*Get_Key) (ArcKAPRec_T), int ArcKey, ArcBST_T &ArcBST) {
	ArcNode_Ptr_T	Arc_Ptr;
	ArcKAPRec_T NewRoot;

	Arc_Ptr = (ArcNode_Ptr_T) ArcBST;

	if (Arc_Ptr != NULL) {

		if (ArcKey < Get_Key(Arc_Ptr->Element)) {
			ArcBST_Delete_R(Get_Key, ArcKey, (ArcBST_T)  Arc_Ptr->Left_Ptr);
		}

		else {
			if (ArcKey > Get_Key(Arc_Ptr->Element)) {
				ArcBST_Delete_R(Get_Key, ArcKey, (ArcBST_T)  Arc_Ptr->Right_Ptr);
			}
			else {
				if (Arc_Ptr->Left_Ptr == NULL) {
					ArcBST = (ArcBST_T)  Arc_Ptr->Right_Ptr;
					delete(Arc_Ptr);
				}
				else {
					if (Arc_Ptr->Right_Ptr == NULL) {
						ArcBST = (ArcBST_T)  Arc_Ptr->Left_Ptr;
						delete(Arc_Ptr);
					}
					else {
						NewRoot = Get_LeftMostArc_R(Arc_Ptr->Right_Ptr);
						Arc_Ptr->Element = NewRoot;
						ArcBST_Delete_R(Get_Key, Get_Key(NewRoot), (ArcBST_T)  Arc_Ptr->Right_Ptr);
					}
				}
			}
		}
	}
}



// PRE	TRUE
// POST	All memory allocated to KAPRec's in ArcBST is freed
void
ArcBST_Dispose_All_KAPRecs_R(ArcBST_T &ArcBST) {
	ArcNode_Ptr_T	Arc_Ptr;

	Arc_Ptr = (ArcNode_Ptr_T) ArcBST;

	if (Arc_Ptr != NULL) {
		ArcBST_Dispose_All_KAPRecs_R((ArcBST_T) Arc_Ptr->Left_Ptr);
		ArcBST_Dispose_All_KAPRecs_R((ArcBST_T)Arc_Ptr->Right_Ptr);
		Free_ArcKAPRecMemory(Arc_Ptr->Element);
	}
}


// PRE	TRUE
// POST	All memory allocated to nodes in ArcBST is freed
void
ArcBST_Dispose_All_Nodes_R(ArcBST_T &ArcBST) {
	ArcNode_Ptr_T	Arc_Ptr;

	Arc_Ptr = (ArcNode_Ptr_T) ArcBST;

	if (Arc_Ptr != NULL) {
		ArcBST_Dispose_All_Nodes_R((ArcBST_T) Arc_Ptr->Left_Ptr);
		ArcBST_Dispose_All_Nodes_R((ArcBST_T) Arc_Ptr->Right_Ptr);
		delete(Arc_Ptr);
	}
}


// PRE	TRUE
// POST	All memory for nodes allocated to ArcBST is freed
//			AND Q_EmptyArcTree(ArcBST)
// NOTES	This will not dispose of memory referenced by the elements
//			e.g. through KAPRec_T's
void
ArcBST_Delete_All(ArcBST_T &ArcBST) {

	ArcBST_Dispose_All_Nodes_R(ArcBST);
	ArcBST = NULL;
}




// PRE	TRUE
// POST	Iterator is created
ArcBST_Iterator_T
Create_ArcIterator() {
	ArcBST_ItRec_Ptr_T ItRec;

	ItRec = new(ArcBST_ItRec_T);
	ItRec->Stack = Create_Stack();
	ItRec->Q_Bound = FALSE;
	return((void *) ItRec);
}


// PRE	Create_Iterator(Iterator) has been called
//			AND since last, if any, call to
//			Destroy_Iterator(Iterator)
// POST	Iterator is destroyed
void
Destroy_ArcIterator(ArcBST_Iterator_T Iterator) {
	ArcBST_ItRec_Ptr_T ItRec;

	ItRec = (ArcBST_ItRec_Ptr_T) Iterator;
	for(;!Q_EmptyStack(ItRec->Stack);) {
		Pop(ItRec->Stack);
	}
	Free_Stack(ItRec->Stack);
}


// PRE	TRUE
// POST	IF Start_Iteration(Iterator, ArcBST) has been executed
//				correctly for some value ArcBST
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
Q_Bound_To_ArcBST(ArcBST_Iterator_T Iterator) {
	ArcBST_ItRec_Ptr_T ItRec;

	ItRec = (ArcBST_ItRec_Ptr_T) Iterator;
	return(ItRec->Q_Bound);
}


// PRE	TRUE
// POST	IF iteration is at ordinally keyed first arc
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_StartOfArcIteration(ArcBST_Iterator_T Iterator) {
	ArcBST_ItRec_Ptr_T ItRec;

	ItRec = (ArcBST_ItRec_Ptr_T) Iterator;
	return(ItRec->CurrentNumber = 1);
}


// PRE	Q_Bound_To_ArcBST(Iterator)
// POST	IF iteration is at ordinally keyed last arc
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EndOfArcIteration(ArcBST_Iterator_T Iterator) {
	ArcBST_ItRec_Ptr_T ItRec;
	ArcNode_Ptr_T CurrentArc;

	ItRec = (ArcBST_ItRec_Ptr_T) Iterator;
	CurrentArc = (ArcNode_Ptr_T) ItRec->Current;
	return(	(CurrentArc->Right_Ptr == NULL) &&
				Q_EmptyStack(ItRec->Stack));
}


// PRE	NOT Q_EmptyTree(ArcBST)
// POST	Iterator is reset with
//			ordinally first key arc as current arc
void
Start_ArcIteration(ArcBST_Iterator_T &Iterator, ArcBST_T ArcBST) {
	ArcBST_ItRec_Ptr_T ItRec;
	ArcNode_Ptr_T Arc_Ptr;

	ItRec = (ArcBST_ItRec_Ptr_T) Iterator;
	for (;!Q_EmptyStack(ItRec->Stack);) {
		 Pop(ItRec->Stack);
	}
	Arc_Ptr = (ArcNode_Ptr_T) ArcBST;
	for (;Arc_Ptr->Left_Ptr != NULL;) {
		Push(Arc_Ptr, ItRec->Stack);
		Arc_Ptr = Arc_Ptr->Left_Ptr;
	}
	// Arc_Ptr is the leftmost arc of ArcBST
	ItRec->Current = (ArcBST_T) Arc_Ptr;
	ItRec->CurrentNumber = 1;
	ItRec->Q_Bound = TRUE;
}


// PRE	Start_Iteration(&Iterator, ArcBST) has been called
//			for some value ArcBST
//			AND NOT Q_EndOfIteration(Iterator)
// POST	NOT Q_Iteration_Start(Iterator') AND
//					current is next node in iteration
void
ArcIterate(ArcBST_Iterator_T &Iterator) {
	ArcBST_ItRec_Ptr_T ItRec;
	ArcNode_Ptr_T Current;

	ItRec = (ArcBST_ItRec_Ptr_T) Iterator;
	(ItRec->CurrentNumber)++;
	Current = (ArcNode_Ptr_T) ItRec->Current;
	if (Current->Right_Ptr == NULL) {
		 ItRec->Current = Top(ItRec->Stack);
		 Pop(ItRec->Stack);
	}
	else {
		 Current = Current->Right_Ptr;
		 for (;(Current->Left_Ptr != NULL);) {
			  Push((ArcBST_T) Current, ItRec->Stack);
			  Current = Current->Left_Ptr;
		 }
		ItRec->Current = (ArcBST_T) Current;
	}
}


// PRE     Q_Iteration_Started(Iterator)
// RETURNS current record in iteration
ArcKAPRec_T
Get_CurrentArc(ArcBST_Iterator_T Iterator) {
	ArcBST_ItRec_Ptr_T ItRec;
	ArcNode_Ptr_T CurrentArc;

	ItRec = (ArcBST_ItRec_Ptr_T) Iterator;
	CurrentArc = (ArcNode_Ptr_T) ItRec->Current;
	return(CurrentArc->Element);
}


// PRE	Q_Iteration_Started(Iterator)
// POST	RETURNS number of current record in iteration
int
Get_CurrentArcNumber(ArcBST_Iterator_T Iterator) {
	ArcBST_ItRec_Ptr_T ItRec;

	ItRec = (ArcBST_ItRec_Ptr_T) Iterator;
	return(ItRec->CurrentNumber);
}


