#include "graph.h"
#include "vstore.h"
#include "arc_bst.h"
#include "stack.h"
#include "rd_str.h"
#include <string.h>


typedef struct {
	VertBST_T	VertBST;
	ArcBST_T		ArcBST,
					FromArcBST;
	VertBST_Iterator_T 	VertIterator;
	ArcBST_Iterator_T 	ArcIterator,
								FromArcIterator;
	int	NumVerts,
			NumArcs;
	int	VirtualInfinity;
}	GraphRec_T, *GraphRec_Ptr_T;


typedef struct {
	VertKey_T	VertKey;
	VertBST_T	VertTreeRoot;
	VertKAPRec_T	CurrentVertKAPRec;
	ArcBST_T		ArcTreeRoot,
					CurrentArcTreeRoot;
	BOOLEAN		Q_NoArcs,
					Q_AtLastArc;
}	TravellerRec_T, *TravellerRec_Ptr_T;


// PRE	TRUE
// POST	RETURNS the empty graph
Graph_T
Create_Graph() {
	GraphRec_Ptr_T	NewGraphRec_Ptr;

	NewGraphRec_Ptr = new(GraphRec_T);
	NewGraphRec_Ptr->VertBST = Create_VertTree();
	NewGraphRec_Ptr->ArcBST = Create_ArcTree();
	NewGraphRec_Ptr->FromArcBST = Create_ArcTree();
	NewGraphRec_Ptr->VertIterator = Create_VertIterator();
	NewGraphRec_Ptr->ArcIterator = Create_ArcIterator();
	NewGraphRec_Ptr->FromArcIterator = Create_ArcIterator();
	NewGraphRec_Ptr->NumVerts = 0;
	NewGraphRec_Ptr->NumArcs = 0;
	NewGraphRec_Ptr->VirtualInfinity = 1;

	return((Graph_T) NewGraphRec_Ptr);
}


//	PRE	TRUE
// POST	Q_EmptyGraph(Graph')
//			AND all memory allocated to Graph is freed up
void
Make_Empty(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;
	VertBST_Dispose_All_KAPRecs_R(GraphRec_Ptr->VertBST);
	VertBST_Delete_All(GraphRec_Ptr->VertBST);
	ArcBST_Dispose_All_KAPRecs_R(GraphRec_Ptr->ArcBST);
	ArcBST_Delete_All(GraphRec_Ptr->ArcBST);
	ArcBST_Delete_All(GraphRec_Ptr->FromArcBST);
}


// PRE	NOT Q_VertPresent(Get_VertKey(Vert), Graph)
// POST	Graph' = Graph UNION {Vert}
//			AND Q_FirstVert(Graph)
void
Add_Vert(VertDataRec_T Vert, Graph_T Graph) {
	VertKAPRec_T	NewKAPRec;
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	NewKAPRec = MakeVertKAPRec(Vert);
	VertBST_Insert_R(Get_VertKAPRecKey, NewKAPRec, GraphRec_Ptr->VertBST);
	GraphRec_Ptr->NumVerts++;
	Start_VertIteration(GraphRec_Ptr->VertIterator, GraphRec_Ptr->VertBST);
}


// PRE	TRUE
// POST	RETURNS VertBST_T containing copies of all verts of Graph
//			ordered by Get_VertKAPRecKey
VertBST_T
Extract_And_Copy_Verts(int (*Get_Key) (VertKAPRec_T),Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Copy_VertTree(Get_Key, GraphRec_Ptr->VertBST));
}

//	PRE	TRUE
// POST	RETURNS number of vertices in Graph
int
Get_Size(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;
	return(GraphRec_Ptr->NumVerts);
}


// PRE	TRUE
// POST	IF Graph is empty
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_EmptyGraph(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Q_EmptyVertTree(GraphRec_Ptr->VertBST));
}


// PRE	NOT Q_EmptyGraph(Graph)
// POST	IF current element of Graph is the first
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_FirstVert(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Q_StartOfVertIteration(GraphRec_Ptr->VertIterator));
}


// PRE	NOT Q_EmptyGraph(Graph)
// POST	IF current element of Graph is the last
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_LastVert(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Q_EndOfVertIteration(GraphRec_Ptr->VertIterator));
}


// PRE	NOT Q_EmptyGraph(Graph)
// POST	current element of Graph is the first
void
Go_To_FirstVert(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;
	Start_VertIteration(GraphRec_Ptr->VertIterator, GraphRec_Ptr->VertBST);
}


// PRE	NOT Q_EmptyGraph(Graph)
//			AND NOT Q_LastVert(Graph)
// POST	current is next node in iteration
void
Go_To_NextVert(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	VertIterate(GraphRec_Ptr->VertIterator);
}


// PRE	NOT Q_EmptyGraph(Graph)
// POST	RETURNS current vertex
VertKAPRec_T
Get_CurrentVertKAPRec(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Get_CurrentVert(GraphRec_Ptr->VertIterator));
}


/*void
Update_CurrentVert(VertKAPRec_T VertKAPRec_T

GraphRec_Ptr->VertIterator);
	Update_CurrentVert(VertKAPRec, GraphRec_Ptr->VertIterator);*/

//	PRE	NOT Q_EmptyGraph(Graph)
//	POST	Q_CurrentVertMarked(Graph')
void
Set_CurrentMark(Graph_T	Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;
	VertKAPRec_T VertKAPRec;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	VertKAPRec = Get_CurrentVert(GraphRec_Ptr->VertIterator);
	Mark_Vert(VertKAPRec);
	Update_CurrentVert(VertKAPRec, GraphRec_Ptr->VertIterator);
}


//	PRE	NOT Q_EmptyGraph(Graph)
//	POST	NOT Q_CurrentVertMarked(Graph')
void
UnSet_CurrentMark(Graph_T	Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;
	VertKAPRec_T VertKAPRec;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	VertKAPRec = Get_CurrentVert(GraphRec_Ptr->VertIterator);
	UnMark_Vert(VertKAPRec);
	Update_CurrentVert(VertKAPRec, GraphRec_Ptr->VertIterator);
}


// PRE	NOT Q_EmptyGraph(Graph)
// POST	IF current vertex is marked
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_CurrentVertMarked(Graph_T	Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;
	return(Q_Marked(Get_CurrentVert(GraphRec_Ptr->VertIterator)));
}


// PRE	TRUE
// POST	IF node with key VertKey in Graph
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
Q_VertPresent(VertKey_T	VertKey, Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Q_VertBST_Present_R(Get_VertKAPRecKey, VertKey, GraphRec_Ptr->VertBST));
}


//	PRE	Q_VertPresent(Vert, Graph)
// POST	RETURNS node with key VertKey from Graph
VertKAPRec_T
Retrieve_Vert(VertKey_T	VertKey, Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(VertBST_Retrieve_R(Get_VertKAPRecKey, VertKey, GraphRec_Ptr->VertBST));
}


// PRE	Q_Vert_Present_R(Get_VertKAPRecKey(VertKAPRecUpdate), Graph)
// POST	record with key value VertKey from VertBST is replaced by VertKAPRecUpdate
void
Update_GraphVert(VertKAPRec_T VertKAPRecUpdate, Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	Update_Vert(VertKAPRecUpdate, GraphRec_Ptr->VertBST);
}



//	PRE	TRUE
//	POST	IF no arcs in Graph
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_NoArcs(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(GraphRec_Ptr->NumArcs == 0);
}


// PRE	NOT Q_ArcPresent(Get_ArcKey(Arc), Graph)
//			AND Q_VertPresent(Get_InitialVertKey(Arc), Graph)
//			AND Q_VertPresent(Get_ToVertKey(Arc), Graph)
// POST	Graph' = Graph UNION {Vert}
//			AND Q_FirstArc(Graph')
void
Add_Arc(ArcDataRec_T Arc, Graph_T Graph) {
	ArcKAPRec_T	NewKAPRec;
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	NewKAPRec = MakeArcKAPRec(Arc);
	ArcBST_Insert_R(Get_ArcKAPRecKey, NewKAPRec, GraphRec_Ptr->ArcBST);
	ArcBST_Insert_R(Get_ArcKAPRecInitialKey, NewKAPRec, GraphRec_Ptr->FromArcBST);
	GraphRec_Ptr->NumArcs++;
	GraphRec_Ptr->VirtualInfinity = GraphRec_Ptr->VirtualInfinity + Get_Weight(Arc);
	Start_ArcIteration(GraphRec_Ptr->ArcIterator, GraphRec_Ptr->ArcBST);
	Start_ArcIteration(GraphRec_Ptr->ArcIterator, GraphRec_Ptr->FromArcBST);
}


//	PRE	TRUE
// POST	RETURNS number of arcs in Graph
int
Get_Number_Of_Arcs(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;
	return(GraphRec_Ptr->NumArcs);
}


// PRE	NOT Q_NoArcs(Graph)
// POST	IF current element of Graph is the first
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_FirstArc(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Q_StartOfArcIteration(GraphRec_Ptr->ArcIterator));
}


// PRE	NOT Q_NoArcs(Graph)
// POST	IF current element of Graph is the last
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_LastArc(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Q_EndOfArcIteration(GraphRec_Ptr->ArcIterator));
}


// PRE	NOT Q_NoArcs(Graph)
// POST	Current arc of Graph is the first
void
Go_To_FirstArc(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	Start_ArcIteration(GraphRec_Ptr->ArcIterator, GraphRec_Ptr->ArcBST);
}


// PRE	NOT Q_NoArcs(Graph)
//			AND NOT Q_LastArc(Graph)
// POST	current is next node in iteration
void
Go_To_NextArc(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	ArcIterate(GraphRec_Ptr->ArcIterator);
}


// PRE	NOT Q_NoArcs(Graph)
// POST	RETURNS current vertex
ArcKAPRec_T
Get_CurrentArcKAPRec(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Get_CurrentArc(GraphRec_Ptr->ArcIterator));
}


// PRE	TRUE
// POST	IF node with key ArcKey in Graph
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
Q_ArcPresent(ArcKey_T	ArcKey, Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(Q_ArcBST_Present_R(Get_ArcKAPRecKey, ArcKey, GraphRec_Ptr->ArcBST));
}


//	PRE	Q_ArcPresent(Arc, Graph)
// POST	RETURNS node with key ArcKey from Graph
ArcKAPRec_T
Retrieve_Arc(ArcKey_T	ArcKey, Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	return(ArcBST_Retrieve_R(Get_ArcKAPRecKey, ArcKey, GraphRec_Ptr->ArcBST));
}


// PRE	Q_VertPresent(VertKey, Graph)
// POST	Traveller_T is created, bound to Graph
//			AND set to vertex with key VertKey
//			AND RETURNed
//			IF NOT Q_Dead_End(Traveller')
//				Current arc of Traveller' is first
Traveller_T
Create_Traveller(VertKey_T	VertKey, Graph_T Graph) {
	TravellerRec_Ptr_T TravellerRec;
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;
	TravellerRec = new TravellerRec_T;
	TravellerRec->VertTreeRoot = GraphRec_Ptr->VertBST;
	TravellerRec->ArcTreeRoot = GraphRec_Ptr->FromArcBST;
	TravellerRec->CurrentArcTreeRoot =
		Find_First_R(Get_ArcKAPRecInitialKey, VertKey, GraphRec_Ptr->FromArcBST);
	TravellerRec->Q_NoArcs = Q_EmptyArcTree(TravellerRec->CurrentArcTreeRoot);
	if (!TravellerRec->Q_NoArcs) {
		TravellerRec->Q_AtLastArc =
			Q_EmptyArcTree(Find_NextDuplicate(Get_ArcKAPRecInitialKey, VertKey, TravellerRec->CurrentArcTreeRoot));
	}
	TravellerRec->VertKey = VertKey;
	return((void *) TravellerRec);
}


//	PRE	TRUE
//	POST	RETURNS key of vertex where Traveller is at.
VertKey_T
Get_TravellerVertKey(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;

	return(TravellerRec->VertKey);
}


// PRE	TRUE
// POST	IF VertKey is on Traveller's graph
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Vert_For_Traveller(Traveller_T Traveller, VertKey_T	VertKey) {

	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;
	return(Q_VertBST_Present_R(Get_VertKAPRecKey, VertKey, TravellerRec->VertTreeRoot));
}




// PRE	Q_Vert_For_Traveller(Traveller, VertKey)
// POST	RETURNS VertKAPRec for VertKey
VertKAPRec_T
Get_Vert_From_Traveller(Traveller_T Traveller, VertKey_T	VertKey) {

	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;
	return(VertBST_Retrieve_R(Get_VertKAPRecKey, VertKey, TravellerRec->VertTreeRoot));
}




// PRE   Q_Vert_For_Traveller(Traveller, VertKey)
// POST	Traveller is reset to vertex with key VertKey
//			IF NOT Q_Dead_End(Traveller')
//				Current arc of Traveller' is first
void
Start_Traveller(Traveller_T Traveller, VertKey_T	VertKey) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;

	TravellerRec->CurrentArcTreeRoot =
		Find_First_R(Get_ArcKAPRecInitialKey, VertKey, 	TravellerRec->ArcTreeRoot);
	TravellerRec->Q_NoArcs = Q_EmptyArcTree(TravellerRec->CurrentArcTreeRoot);
	if (!TravellerRec->Q_NoArcs) {
		TravellerRec->Q_AtLastArc =
			Q_EmptyArcTree(Find_NextDuplicate(Get_ArcKAPRecInitialKey, VertKey, TravellerRec->CurrentArcTreeRoot));
	}
	TravellerRec->VertKey = VertKey;
}


// PRE   Q_Vert_For_Traveller(Traveller, VertKey)
//			AND VertKey is the key of the initial vertex of the
//				arc whose key is ArcKey
// POST	Traveller is reset to vertex with key VertKey
//			IF NOT Q_Dead_End(Traveller')
//			Current arc of Traveller' is that with key ArcKey
void
ReSet_Traveller(Traveller_T Traveller, VertKey_T	VertKey, ArcKey_T ArcKey) {
	TravellerRec_Ptr_T TravellerRec;
	ArcBST_T		ArcBST;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;

	ArcBST =
		Find_First_R(Get_ArcKAPRecInitialKey, VertKey, 	TravellerRec->ArcTreeRoot);
	for (;ArcKey != Get_ArcKAPRecKey(Arc_Get_Root(ArcBST));) {
		ArcBST =
			Find_NextDuplicate(Get_ArcKAPRecInitialKey, VertKey, ArcBST);
	}
	TravellerRec->CurrentArcTreeRoot = ArcBST;
	TravellerRec->Q_NoArcs = FALSE;
	TravellerRec->Q_AtLastArc =
		Q_EmptyArcTree(Find_NextDuplicate(Get_ArcKAPRecInitialKey, VertKey, ArcBST));
	TravellerRec->VertKey = VertKey;
}



//	PRE	TRUE
//	POST	IF Traveller is at the end of a path
//			i.e. there are no further arcs forward
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Dead_End(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;
	return(TravellerRec->Q_NoArcs);
}


//	PRE	NOT Q_Dead_End(Traveller)
//	POST	Current arc of Traveller' is the first
void
Go_To_FirstArc_From_Vertex(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;
	TravellerRec->CurrentArcTreeRoot =
			Find_First_R(Get_ArcKAPRecInitialKey, TravellerRec->VertKey, TravellerRec->ArcTreeRoot);
	TravellerRec->Q_AtLastArc =
			Q_EmptyArcTree(Find_NextDuplicate(Get_ArcKAPRecInitialKey, TravellerRec->VertKey, TravellerRec->CurrentArcTreeRoot));
}


//	PRE	NOT Q_Dead_End(Traveller) AND NOT Q_No_More_Arcs(Traveller)
//	POST	Current arc of Traveller' is next one after that of Traveller
void
Go_To_NextArc_From_Vertex(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;
	TravellerRec->CurrentArcTreeRoot =
			Find_NextDuplicate(Get_ArcKAPRecInitialKey, TravellerRec->VertKey, TravellerRec->CurrentArcTreeRoot);
	TravellerRec->Q_AtLastArc =
			Q_EmptyArcTree(Find_NextDuplicate(Get_ArcKAPRecInitialKey, TravellerRec->VertKey, TravellerRec->CurrentArcTreeRoot));
}


//	PRE	TRUE
// POST	RETURNS VertKAPRec_T with Get_TravellerVertKey(Traveller)
VertKAPRec_T
Get_TravellerVertKAPRec(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;

	return(VertBST_Retrieve_R(Get_VertKAPRecKey, TravellerRec->VertKey, TravellerRec->VertTreeRoot));
}


//	PRE	TRUE
//	POST	IF current arc in Traveller is last
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_No_More_Arcs(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;
	return(TravellerRec->Q_AtLastArc);
}


//	PRE	NOT Q_Dead_End(Traveller)
//	POST	Traveller' has moved along current arc of Traveller
//			AND IF NOT Q_Dead_End(Traveller)
//				current arc of Traveller' is the first
void
Move_To_Vertex_Along_Current_Arc(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;
	TravellerRec->VertKey = Get_ArcKAPRecFinalKey(Arc_Get_Root(	TravellerRec->CurrentArcTreeRoot));
	TravellerRec->CurrentArcTreeRoot =
		Find_First_R(Get_ArcKAPRecInitialKey, TravellerRec->VertKey, TravellerRec->ArcTreeRoot);
	TravellerRec->Q_NoArcs = Q_EmptyArcTree(TravellerRec->CurrentArcTreeRoot);
	if (!TravellerRec->Q_NoArcs) {
		TravellerRec->Q_AtLastArc =
			Q_EmptyArcTree(Find_NextDuplicate(Get_ArcKAPRecInitialKey, TravellerRec->VertKey, TravellerRec->ArcTreeRoot));
	}
}


//	PRE	NOT Q_No_More_Arcs(Traveller)
//	POST	RETURNS Current arc for Traveller
ArcKAPRec_T
Get_CurrentTraveller_ArcKAPRec(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;

	return(Arc_Get_Root(TravellerRec->CurrentArcTreeRoot));
}


//	PRE	TRUE
//	POST	Vertex for Traveller' is marked
void
Set_TravellerMark(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;
	VertKAPRec_T VertKAPRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;

	VertKAPRec = VertBST_Retrieve_R(Get_VertKAPRecKey, TravellerRec->VertKey, TravellerRec->VertTreeRoot);
	Mark_Vert(VertKAPRec);
	Update_Vert(VertKAPRec, TravellerRec->VertTreeRoot);
}


//	PRE	TRUE
//	POST	Vertex for Traveller' is unmarked
void
UnSet_TravellerMark(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;
	VertKAPRec_T VertKAPRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;

	VertKAPRec = VertBST_Retrieve_R(Get_VertKAPRecKey, TravellerRec->VertKey, TravellerRec->VertTreeRoot);
	UnMark_Vert(VertKAPRec);
	Update_Vert(VertKAPRec, TravellerRec->VertTreeRoot);
}


//	PRE	TRUE
//	POST	IF Vertex with key VertKey is marked for Traveller
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_VertMarked_For_Traveller(VertKey_T VertKey, Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;

	return(Q_Marked(VertBST_Retrieve_R(Get_VertKAPRecKey, VertKey, TravellerRec->VertTreeRoot)));
}


// PRE	Create_ArcIterator(Traveller) has been called
//			AND since last, if any, call to
//			Destroy_Iterator(Iterator)
// POST	Memory allocated to Traveller is freed
void
Destroy_Traveller(Traveller_T Traveller) {
	TravellerRec_Ptr_T TravellerRec;

	TravellerRec = (TravellerRec_Ptr_T) Traveller;
	delete TravellerRec;
}


// PRE	TRUE
// POST	User is prompted for filename
//			IF this is for a pair of files containing valid vertices and arcs for a graph
//				Graph is emptied and then contents of files are loaded into Graph
//			ELSE
//				Q_EmptyGraph(Graph')
void
Load_Graph_From_File(Graph_T Graph) {
	char	*GraphId,
			*GraphVertsFN,
			*GraphArcsFN;
	ifstream	InFile;
	BOOLEAN	Q_OKRead;
	GraphRec_Ptr_T	GraphRec_Ptr;
	VertDataRec_T	VertDataRec;
	ArcDataRec_T	ArcDataRec;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;

	GraphRec_Ptr->NumVerts = 0;
	GraphRec_Ptr->NumArcs = 0;
	VertBST_Dispose_All_KAPRecs_R(GraphRec_Ptr->VertBST);
	VertBST_Delete_All(GraphRec_Ptr->VertBST);
	ArcBST_Dispose_All_KAPRecs_R(GraphRec_Ptr->ArcBST);
	ArcBST_Delete_All(GraphRec_Ptr->ArcBST);
	ArcBST_Delete_All(GraphRec_Ptr->FromArcBST);

	cout	<< "\nPlease type identifier of graph ==> ";
	GraphId = Read_String();

	Q_OKRead = TRUE;
	GraphVertsFN = new char (strlen(GraphId) + 1);
	strcpy(GraphVertsFN, GraphId);
	strcat(GraphVertsFN, ".ver");
	InFile.open(GraphVertsFN, ios::in);

	if (InFile == NULL) {
		cout	<< "\n\nError - cannot open "
				<< GraphVertsFN
				<< "\n\n";
		Q_OKRead = FALSE;
	}

	if (Q_OKRead) {
		cout	<< "\nLoading vertices from "
				<< GraphVertsFN;
		for(;(!InFile.eof()) && Q_OKRead;) {
			VertDataRec = Read_VertDataRec_From_File(InFile, Q_OKRead);
			cout	<< "\n";
			if (!InFile.eof()) {
				if (!Q_OKRead) {
					cout	<< "\n\nError reading from"
							<< GraphVertsFN
							<<	"\nRolling back ....";
				}
				else {
					if (Q_VertPresent(Get_VertKey(VertDataRec), Graph)) {
						cout	<<	"\n\nVertex with key ";
						Write_VertKey(Get_VertKey(VertDataRec));
						cout	<< " already in graph - rolling back....";
						Q_OKRead = FALSE;
					}
					else {
						Write_VertDataRec(VertDataRec);
						Add_Vert(VertDataRec, Graph);
					}
				}
			}
		}
		InFile.close();
	}
	if (Q_OKRead) {
		GraphArcsFN = new char (strlen(GraphId) + 1);
		strcpy(GraphArcsFN, GraphId);
		strcat(GraphArcsFN, ".arc");
		InFile.open(GraphArcsFN, ios::in);

		if (InFile == NULL) {
			cout	<< "\n\nError - cannot open "
					<< GraphArcsFN
					<< "\n\n";
			Q_OKRead = FALSE;
		}

		if (Q_OKRead) {
			cout	<< "\nLoading arcs from "
					<< GraphArcsFN;
			ArcDataRec = Read_ArcDataRec_From_File(InFile, Q_OKRead);
			for(;(!InFile.eof()) && Q_OKRead;) {
				if (Q_ArcPresent(Get_ArcKey(ArcDataRec), Graph)) {
					cout	<<	"\nArc with key ";
					Write_ArcKey(Get_ArcKey(ArcDataRec));
					cout	<<	" is already in graph\n";
				}
				else {
					if (!Q_VertPresent(Get_InitialVertKey(ArcDataRec), Graph)) {
						cout	<<	"\nFrom vertex with key ";
						Write_VertKey(Get_InitialVertKey(ArcDataRec));
						cout	<<	" is not in graph\n";
					}
					else {
						if (!Q_VertPresent(Get_FinalVertKey(ArcDataRec), Graph)) {
							cout	<<	"\nTo vertex with key ";
							Write_VertKey(Get_FinalVertKey(ArcDataRec));
							cout	<<	" is not in graph\n";
						}
						else {
							Add_Arc(ArcDataRec, Graph);
                     Write_ArcDataRec(ArcDataRec);
						}
					}
				}
				if (!InFile.eof() && Q_OKRead) {
					ArcDataRec = Read_ArcDataRec_From_File(InFile, Q_OKRead);
				}
			}
		}
	}

	if (!Q_OKRead) {
		GraphRec_Ptr->NumVerts = 0;
		GraphRec_Ptr->NumArcs = 0;
		VertBST_Dispose_All_KAPRecs_R(GraphRec_Ptr->VertBST);
		VertBST_Delete_All(GraphRec_Ptr->VertBST);
		ArcBST_Dispose_All_KAPRecs_R(GraphRec_Ptr->ArcBST);
		ArcBST_Delete_All(GraphRec_Ptr->ArcBST);
		ArcBST_Delete_All(GraphRec_Ptr->FromArcBST);
	}
	else {
		cout	<<	"\nGraph "
				<< GraphId
				<<	" loaded successfully\n\n";
	}
}

// **************************************************
// START Shortest path operations
// **************************************************

// PRE	TRUE
// POST	RETURNS sum of all path weights in Graph plus 1
int
Get_VirtualInfinity(Graph_T Graph) {
	GraphRec_Ptr_T	GraphRec_Ptr;

	GraphRec_Ptr = (GraphRec_Ptr_T) Graph;
	return(GraphRec_Ptr->VirtualInfinity);
}

// **************************************************
// END Shortest path operations
// **************************************************


