// Ken Brownsey
// 31/8/98
// bst_util.cpp
// BST_T utilities

#include "bst_util.h"
#include "rd_str.h"

typedef enum {LeftTurn, RightTurn} Turn_T;

typedef int Length_T;

// PRE	TRUE
// POST	BST is printed inorder
void
Print_Tree_R(BST_T BST) {
	if (Q_EmptyTree(BST)) {
		cout	<< "()";
	}
	else {
		cout	<< "(";
		Print_Tree_R(Get_Left(BST));
		Write_Rec(Get_Root(BST));
		Print_Tree_R(Get_Right(BST));
		cout	<< ")";
	}
}


// PRE	TRUE
// POST	Elements of BST are printed inorder
void
InOrder_R(BST_T BST) {
	if (!Q_EmptyTree(BST)) {
		InOrder_R(Get_Left(BST));
		Write_Rec(Get_Root(BST));
		cout	<< "\n";
		InOrder_R(Get_Right(BST));
	}
}


// PRE	TRUE
// POST	Elements of BST are printed preorder
void
PreOrder_R(BST_T BST) {
	if (!Q_EmptyTree(BST)) {
		Write_Rec(Get_Root(BST));
		cout	<< "\n";
		PreOrder_R(Get_Left(BST));
		PreOrder_R(Get_Right(BST));
	}
}


// PRE	TRUE
// POST	Elements of BST are printed postorder
void
PostOrder_R(BST_T BST) {
	if (!Q_EmptyTree(BST)) {
		PostOrder_R(Get_Left(BST));
		PostOrder_R(Get_Right(BST));
		Write_Rec(Get_Root(BST));
		cout	<< "\n";
	}
}


// PRE	TRUE
// POST	User is prompted for filename
//			IF this is a file of Element_T's
//				these are loaded into DBase
//				except for those already there
//				which are printed with appropriate message
void
Load_Tree_From_File(BST_T &BST) {
	char	*FName;
	ifstream	InFile;
	Element_T NextRec;
	BOOLEAN	Q_OKRead;

	cout	<< "\nPlease type name of file to load from ==> ";
	FName = Read_String();
	InFile.open(FName, ios::in);
	if (InFile == NULL) {
		cout	<< "\n\nError - cannot open "
				<< FName
				<< "\n\n";
	}
	else {
		cout	<< "\nLoading records from "
				<< FName;
		for(;!InFile.eof();) {
			NextRec = Read_Rec_From_File(InFile, Q_OKRead);
			if (Q_OKRead) {
				if (!InFile.eof()) {
					if (!Q_BST_Present_R(Get_Key(NextRec), BST)) {
						cout	<< "\nRecord ==> ";
						Write_Rec(NextRec);
						BST_Insert_R(NextRec, BST);
					}
					else {
						cout	<< "\nRecord with key ";
						Write_Key(Get_Key(NextRec));
						cout	<< " is already in database";
					}
				}
			}
			else {
				cout	<<	"\nThere is a problem reading from file "
						<<	FName
						<<	"\n\n";
			}
		}
	}
}


// PRE	TRUE
// POST	Records in Tree written to OutFile
void
Save_Aux_R(BST_T Tree, ofstream	&OutFile) {

	if (!Q_EmptyTree(Tree)) {
  		Write_DataRec_To_File(Get_Root(Tree), OutFile);
		Save_Aux_R(Get_Left(Tree), OutFile);
		Save_Aux_R(Get_Right(Tree), OutFile);
	}
}


// PRE	TRUE
// POST	User is prompted for filename
//		IF file already exists
//			user is asked whether to overwrite or not
//			IF yes
//				BST is saved to file
//			ELSE
//				message
//		ELSE
//			BST is saved to newly created file
void
Save_Tree_To_File(BST_T BST) {
	char	*FName;
	char	Response[3];
	ofstream	OutFile;
	BOOLEAN	Q_GoAhead,
				Q_OK;

	cout	<<	"Please type name of file to save to => ";
	FName = Read_String();
	Q_GoAhead = TRUE;
/*	if	(FIO.Exists(FName)) {
		cout	<<	FName
				<<	"\n exists already - overwrite yes/no\n- full word please ==> ";
		Response = Read_String();
		if (strcmp(Response, "yes") <> 0) {
			cout	<<	"\nAborting save";
			Q_GoAhead = FALSE;
		}
		else {*/
			OutFile.open(FName, ios::out);
			if (OutFile == NULL) {
				Q_GoAhead = FALSE;
				cout	<<	"Error - cannot open "
						<<	FName
						<<	"\n";
			}
/*		}
	}
	ELSE
		OutFile = FIO.Create(FName)
	END;*/
	if (Q_GoAhead) {
		cout	<<	("Saving records to ")
				<<	FName
				<<	"\n";
		Save_Aux_R(BST, OutFile);
		OutFile.close();
	}
}



// PRE TRUE
// POST RETURNS size of BST
int
Size_R(BST_T BST) {
	if (Q_EmptyTree(BST)) {
		return(0);
	}
	else {
		return(Size_R(Get_Left(BST)) + 1 + Size_R(Get_Right(BST)));
	}
}


// PRE NOT Q_Empty(BST)
// POST RETURNS sum of keys in BST
Key_T
Sum_R(BST_T BST) {
	if (Q_EmptyTree(Get_Left(BST)) && Q_EmptyTree(Get_Right(BST))) {
		return(Get_Key(Get_Root(BST)));
	}
	if (Q_EmptyTree(Get_Left(BST))) {
		return(Get_Key(Get_Root(BST)) + Sum_R(Get_Right(BST)));
	}
	if (Q_EmptyTree(Get_Right(BST))) {
		return(Get_Key(Get_Root(BST)) + Sum_R(Get_Left(BST)));
	}
	return(Get_Key(Get_Root(BST)) + Sum_R(Get_Left(BST)) + Sum_R(Get_Right(BST)));
}


//	PRE	Path >= 0
// POST	Path' is integer representing path
//			represented by Path with further Turn
// NOTES	Path needs a depth to be used because of leading zeros
void
Extend_Path(Path_T &Path, Turn_T Turn) {
	if (Turn == LeftTurn) {
		Path = Path*2;
	}
	else {
		Path = Path*2 + 1;
	}
}

// PRE	Num > 1 AND Exp >=0
// POST	RETURNS Num raised to Exp
//			e.g. Power(5, 3) RETURNS 125
int
Power(int Num, int Exp) {
	if (Exp == 0) {
		return(1);
	}
	else {
		return(Num * Power(Num, Exp - 1));
	}
}



//	PRE	Path < 2^Length - i.e Path represents a binary
//			path of length Length
// POST	Turn' is the next turn in Path
//			Path' is Path with Turn' removed
//			Length' = Length - 1
void
Next_Turn(Path_T &Path, Turn_T &Turn, Length_T &Length) {
	int Chunk;

	Chunk = Power(2, Length - 1);
	if (Chunk > Path) {
		Turn = LeftTurn;
	}
	else {
		Turn = RightTurn;
		Path = Path - Chunk;
	}
	Length--;
}


// PRE	TRUE
// POST	User is prompted for Path and Length
//			IF Path >= 0 AND Length > 0 AND Path < 2^Length
//				path represented by Path and Length
//				is printed out
void
Show_Path() {
	Path_T Path;
	Length_T Length;
	Turn_T Turn;

	cout	<< "\nType in integer path == > ";
	cin	>>	Path;
	cout	<< "\nType in integer length == > ";
	cin	>>	Length;

	for (; Length > 0;) {
		Next_Turn(Path, Turn, Length);
		if (Turn == LeftTurn) {
			cout	<< "L ";
		}
		else {
			cout	<< "R ";
		}
	}
}


// PRE	TRUE
// POST  RETURNS maximum of X and Y
int
Maximum(int X, int Y) {
	if (X > Y) {
		return(X);
	}
	else {
		return(Y);
	}
}


// PRE	TRUE
// POST	RETURNS depth of Tree
Depth_T
Find_Depth_R(BST_T Tree) {
	Depth_T 	LeftDepth,
				RightDepth;

	if(Q_EmptyTree(Tree)) {
		return(0);
	}
	LeftDepth = Find_Depth_R(Get_Left(Tree));
	RightDepth = Find_Depth_R(Get_Right(Tree));
	return(1 + Maximum(LeftDepth, RightDepth));
}


// PRE	Path >= 0 AND Length >= 0
// POST	Prints out bars corresponding to parts of connections
//			in binary tree for element or empty tree whose position
//			in tree is given by Path and Length
// NOTE	Path of 0 length corresponds to one element tree
//			or empty tree
void
Print_Bars(Path_T &Path, Length_T &Length) {
	Turn_T	LastTurn,
				NextTurn;

	if (Length > 0) {
		Next_Turn(Path, LastTurn, Length);
		cout	<< "     ";
		for (; Length > 0;) {
			Next_Turn(Path, NextTurn, Length);
			if (NextTurn == LastTurn) {
				cout	<< "     ";
			}
			else {
				cout	<< "|    ";
			}
			LastTurn = NextTurn;
		}
	}
}


// PRE	Path >= 0, Depth >= 1
// POST	Prints out keys of Tree sideways at Depth
// 		where Depth - 1 is
//			number of levels of Tree from root
void
Tree_Diagram_R(BST_T Tree, Path_T Path, Depth_T Depth) {
	Length_T	Length;
	Path_T	LeftPath,
				RightPath;

	Length = Depth - 1;
	if (Q_EmptyTree(Tree)) {
		Print_Bars(Path, Length);
		cout << "\n";
	}
	else {
		LeftPath = Path;
		RightPath = Path;
		Extend_Path(LeftPath, LeftTurn);
		Tree_Diagram_R(Get_Left(Tree), LeftPath, Depth + 1);
		Print_Bars(Path, Length);
		cout	<< "|--->"
				<< Get_Key(Get_Root(Tree))
				<< "\n";
		Extend_Path(RightPath, RightTurn);
		Tree_Diagram_R(Get_Right(Tree), RightPath, Depth + 1);
	}
}



