// board.c
// demo program for ADT's
// Ken Brownsey 25/8/97

#include <iostream.h>
#include "board.h"

typedef struct {
	enum SquareStatus_T Board[NumRows][NumCols];
} BoardStruct_T, *BoardStruct_Ptr_T;

typedef struct {
	Row_T	Up;
	Col_T Along;
}	Move_T;

Move_T	Moves[8] = {{-1, 0}, {-1, 1}, {0, 1}, {1, 1},
							{1, 0}, {1, -1}, {0, -1}, {-1, -1}};

enum Direction_T	Directions[9] =
	{N, NE, E, SE, S, SW, W, NW, Invalid};

typedef char *DirectionName_T;

DirectionName_T DirectionNames[9] =
	{"North", "North East", "East",
	 "South East", "South", "South West",
	 "West",  "North West", "Invalid Direction"};

// PRE	TRUE 
// POST	RETURNS an empty Board_T - i.e. memory allocated etc. 
Board_T
Create_Board() {

	return ((Board_T) new (BoardStruct_T));
}


//	PRE 	TRUE
//	POST	RETURNS IF 	Position is on Board THEN TRUE
//			ELSE FALSE
BOOLEAN
Q_OnBoard(Position_T Position) {

	return(	(Position.Row >= 1) && (Position.Row <= NumRows) &&
				(Position.Col >= 1) && (Position.Col <= NumCols));
}

//	PRE		Q_OnBoard(Position, Board) 
//	POST 	RETURNS content of Position on Board
enum SquareStatus_T
Get_Status(Position_T Position, Board_T Board) {
	BoardStruct_Ptr_T	Bd;

	Bd = (BoardStruct_Ptr_T) Board;
	return(Bd->Board[Position.Row - 1][Position.Col - 1]);
}


//	PRE		TRUE 
//	POST	RETURNS position in direction Direction from Position 
Position_T
Get_NextPosition(Position_T Position, enum Direction_T Direction) {
	Position_T	NextPosition;

	NextPosition.Row = Position.Row + Moves[Direction].Up;
	NextPosition.Col = Position.Col + Moves[Direction].Along;
	return(NextPosition);
}


//	PRE		Q_OnBoard(Position, Board)
//	POST    content of Position on Board is Status
void
Set_Status(Position_T Position, enum SquareStatus_T Status, Board_T Board) {
	BoardStruct_Ptr_T	Bd;

	Bd = (BoardStruct_Ptr_T) Board;

	Bd->Board[Position.Row - 1][Position.Col - 1] = Status;
}

//	PRE     TRUE 
//	POST	RETURNS point with first coordinate Row and second Col 
Position_T
Make_Position(int Row, int Col) {
	Position_T	Result;

	Result.Row = Row;
	Result.Col = Col;
	return(Result);
}


// PRE	TRUE 
// POST	IF Position is that of a black square
//			RETURNS True
//		ELSE
//			RETURNS False
BOOLEAN
Q_BlackSquare(Position_T Position) {
	return((Position.Row + Position.Col)%2 == 0);
}


//	PRE	TRUE
// POST	IF Position1 and Position2 are the same THEN
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_SamePosition(Position_T Position1, Position_T Position2) {
	return(	(Position1.Row == Position2.Row) &&
				(Position1.Col == Position2.Col));
}


// PRE  TRUE
// POST Prompts for and RETURNS a Position_T
Position_T
Read_Position() {
	Row_T Row;
	Col_T	Col;

	cout	<< "\nType in row number of piece ==> ";
	cin 	>> Row;
	cout 	<< "\nType in column number of piece ==> ";
	cin	>> Col;
	return(Make_Position(Row, Col));
}


//	PRE	TRUE 
// POST	Position printed in Cartesian coordinate format to screen 
void
Write_Position(Position_T Position) {
	cout	<< "("
			<< Position.Row
			<< ", "
			<< Position.Col
			<< ")";
}


// PRE  TRUE
// POST Prompts for and RETURNS a Direction_T
//		  IF the Direction_T read is not valid
//				RETURNS(Invalid)
Direction_T
Read_Direction() {
	int	NumDirection;

	cout 	<< "\nNW  N  NE";
	cout 	<< "\n  8 1 2";
	cout 	<< "\nW 7   3 E";
	cout 	<< "\n  6 5 4";
	cout 	<< "\nSW  S  SE";
	cout 	<< "\n\nType in direction to move ==> ";
	cin	>> NumDirection;
	if ((NumDirection >= 1) && (NumDirection <= 8)) {
		return(Directions[NumDirection - 1]);
	}
	else {
		return(Invalid);
	}
}


//	PRE	TRUE
// POST	Direction printed as string to screen
void
Write_Direction(Direction_T Direction) {
	cout	<< DirectionNames[Direction];
}


// PRE	TRUE
// POST	IF Direction is a valid one
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_ValidDirection(Direction_T Direction) {
	return(Direction != Invalid);
}


//	PRE	TRUE
// POST	RETURNS random valid Direction_T
Direction_T
Random_Direction() {
	int	NumDirection;

	NumDirection = 1 + random(4) * 2;
	return(Directions[NumDirection]);
}
