// data.cpp
// 6/8/98
// data.cpp - token records for parsing and evaluating
// Ken Brownsey

#include "data.h"
#include <iostream.h>

typedef 	union {
		char	CharVal;
		int	IntVal;
	} TokVal_T;

typedef struct {
	TokenType_T	TokenType;
	TokVal_T TokVal;
}	ArithTokenRec_T, *ArithTokenRecPtr_T;


// PRE	TRUE
// POST	IF last lex produced valid arithmetic token
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
Q_ValidAT() {
	TokenType_T TT;

	TT = Get_TokenType();

	if (Q_LexOK()) {
		if (	(TT == LRB_TT) ||
				(TT == RRB_TT) ||
				(TT == Operator_TT) ||
				(TT == INT_TT)	||
				(TT == EndOfLine_TT))				 {
			return(TRUE);
		}
		else {
			return(FALSE);
		}
	}
	else {
		return(FALSE);
	}
}


//	PRE	TRUE
// POST	Read_Rec can be used
// NOTES	This flushes any left over from standard cin operations
//			any number of Read_Rec's can be performed until the next regular
//			cin operation
void
Flush_For_Read() {
	Start_Lex();
	GoTo_NewLine();
	Start_NewLine();
}


// PRE	TRUE
//	POST	Reads up to the next newline
// NOTES to be used when Q_LexOK() is FALSE
void
Parse_Error_Recovery() {
	GoTo_NewLine();
}


// PRE	Flush_For_Read() has been executed since last regular cin operation
// POST	IF next token read from standard input is a valid arithmetic token
//				ArithToken has that value
//				AND Q_ReadOK is TRUE
//			ELSE
//				Q_ReadOK is FALSE
void
Read_Rec(ArithToken_T &ArithToken, BOOLEAN &Q_ReadOK){
	ArithTokenRecPtr_T NewArithToken;
	TokenType_T	TT;

	Lex_Token();
	Skip_Separators();

	Q_ReadOK = Q_ValidAT();
	if (Q_ReadOK) {
		NewArithToken = new ArithTokenRec_T;
		TT = Get_TokenType();

		NewArithToken->TokenType = TT;
		switch(TT) {
			case Operator_TT:
				NewArithToken->TokVal.CharVal = Get_CharVal();
			break;
			case INT_TT:
				NewArithToken->TokVal.IntVal = Get_IntVal();
			break;
			default:
			break;
		}
		ArithToken = (ArithToken_T) NewArithToken;
	}
}


// PRE	Q_ValidAT()
// POST	Writes value of ArithToken_T to screen
void
Write_Rec(ArithToken_T ArithToken) {
	ArithTokenRecPtr_T Tok;

	Tok = (ArithTokenRecPtr_T) ArithToken;
	switch(Tok->TokenType) {
		case LRB_TT:
			cout	<< '(';
		break;
		case RRB_TT:
			cout	<< ')';
		break;
		case Operator_TT:
			cout	<< Tok->TokVal.CharVal;
		break;
		case INT_TT:
			cout	<< Tok->TokVal.IntVal;
		break;
		default:
		break;
	}
}



// PRE	TRUE
// POST	IF ArithToken is LRB
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_LRB(ArithToken_T ArithToken) {
	ArithTokenRecPtr_T Tok;

	Tok = (ArithTokenRecPtr_T) ArithToken;
	return(Tok->TokenType == LRB_TT);
}


// PRE	TRUE
// POST	IF ArithToken is RRB
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_RRB(ArithToken_T ArithToken) {
	ArithTokenRecPtr_T Tok;

	Tok = (ArithTokenRecPtr_T) ArithToken;
	return(Tok->TokenType == RRB_TT);
}


// PRE	TRUE
// POST	IF ArithToken is Operator
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Op(ArithToken_T ArithToken) {
	ArithTokenRecPtr_T Tok;

	Tok = (ArithTokenRecPtr_T) ArithToken;
	return(Tok->TokenType == Operator_TT);
}


// PRE	TRUE
// POST	IF ArithToken is an integer
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_Int(ArithToken_T ArithToken)  {
	ArithTokenRecPtr_T Tok;

	Tok = (ArithTokenRecPtr_T) ArithToken;
	return(Tok->TokenType == INT_TT);
}


// PRE	TRUE
// POST	IF ArithToken is an new line
//				RETURNS TRUE
//			ELSE
//				RETURNS FALSE
BOOLEAN
Q_NL(ArithToken_T ArithToken)  {
	ArithTokenRecPtr_T Tok;

	Tok = (ArithTokenRecPtr_T) ArithToken;
	return(Tok->TokenType == EndOfLine_TT);
}


// PRE	Q_Op(ArithToken)
// POST	RETURNS precedence of operator in ArithToken
Precedence_T
Get_Precedence(ArithToken_T ArithToken) {
	ArithTokenRecPtr_T Tok;
	Precedence_T Prec;

	Tok = (ArithTokenRecPtr_T) ArithToken;
	switch(Tok->TokVal.CharVal) {
		case Minus_C:
		case Plus_C:
			Prec = 1;
		break;
		case Mult_C:
		case Div_C:
			Prec = 2;
		break;
		default:
		break;
	}
	return(Prec);
}


// PRE	Q_Int(AT1) AND Q_Op(AT2) AND Q_Int(AT3)
// POST	RETURNS result of (infix) AT1 AT2 AT3
ArithToken_T
Evaluate(ArithToken_T AT1, ArithToken_T AT2, ArithToken_T AT3) {
	ArithTokenRecPtr_T Tok1,
							Tok2,
							Tok3;

	ArithTokenRecPtr_T NewArithToken;

	Tok1 = (ArithTokenRecPtr_T) AT1;
	Tok2 = (ArithTokenRecPtr_T) AT2;
	Tok3 = (ArithTokenRecPtr_T) AT3;

	NewArithToken = new ArithTokenRec_T;
	NewArithToken->TokenType = INT_TT;

	switch(Tok2->TokVal.CharVal) {
		case '+':
			NewArithToken->TokVal.IntVal =
				Tok1->TokVal.IntVal + Tok3->TokVal.IntVal;
		break;
		case '-':
			NewArithToken->TokVal.IntVal =
				Tok1->TokVal.IntVal - Tok3->TokVal.IntVal;
		break;
		case '/':
			NewArithToken->TokVal.IntVal =
				Tok1->TokVal.IntVal / Tok3->TokVal.IntVal;
		break;
		case '*':
			NewArithToken->TokVal.IntVal =
				Tok1->TokVal.IntVal * Tok3->TokVal.IntVal;
		break;
		default:
		break;
	}
	return(NewArithToken);
}


//	PRE	-	TRUE
//	POST	- 	all separator tokens skipped until next non-separator token,
//				which is then the current token
void
Skip_Separators() {
	for(;Get_TokenType() == Separator_TT;) {
		Lex_Token();
	}
}



