// knapdemo.cpp
// 23/8/98
// Demonstrates recursive solution to simplified knapsack problem
// Ken Brownsey

#include "rec_util.h"
#include <stdlib.h>
#define MaxWeight 10

typedef Element_T Weight_T;
typedef Stack_T Weights_T;
typedef struct {
	BOOLEAN Q_Success;
	Weights_T Weights;
} KnapResult_T;


// PRE	TRUE
// POST  Prompts for number of weights
//			AND sets up random set of weights
void
Set_Up_Weights(Weights_T Weights) {
	int	NumWeights,
			Index;
	Weight_T NextWeight,
				TotalWeight;
	int	Seed;

	for(;!(Q_EmptyStack(Weights));) {
		Pop(Weights);
	}

	cout	<< "\nHow many weights ==> ";
	cin	>> NumWeights;

	cout	<< "\nPlease type in any number ==> ";
	cin	>> Seed;
	srand(Seed);
	TotalWeight = 0;
	for(Index = 1; Index <= NumWeights; Index++) {
			NextWeight = rand() % MaxWeight + 1;
			TotalWeight = TotalWeight + NextWeight;
			Push(NextWeight, Weights);
	}
	cout	<< "\n Weights as follows \n";
	Print_Stack_R(Weights);
	cout	<< "\n Total weight is "
			<< TotalWeight;
}

// PRE	TRUE
// POST	IF elements of Weights add to Target
//				RETURNS 	those elements in Weights field
//							with Q_Success field set to TRUE
//			ELSE
//				RETURNS	with Q_Success field set to FALSE
KnapResult_T
Do_Knap_R(Weight_T Target, Weights_T Weights) {
	Weight_T		NextTarget,
					NextWeight;
	KnapResult_T 	Result,
						SolWith,
						SolWithout;
	cout	<< "\nTarget is "
			<< Target
			<< " Weights are ";
	Print_Stack_R(Weights);

	if (Target == 0) {
		Result.Weights = Create_Stack();
		Result.Q_Success = TRUE;
		return(Result);
	}

	if (Target < 0) {
		Result.Q_Success = FALSE;
		return(Result);
	}

	if (Q_EmptyStack(Weights)) {
			Result.Q_Success = FALSE;
		return(Result);
	}

	NextWeight = Top(Weights);
	Pop(Weights);

	NextTarget = Target - NextWeight;
	SolWith = Do_Knap_R(NextTarget, Weights);
	if (SolWith.Q_Success) {
		Push(NextWeight, SolWith.Weights);
		return(SolWith);
	}

	SolWithout = Do_Knap_R(Target, Weights);
	if (SolWithout.Q_Success) {
		return(SolWithout);
	}

	Push(NextWeight, Weights);
	Result.Q_Success = FALSE;
	return(Result);
}


// PRE	TRUE
// POST  Prompts for target weight
//			AND prints result of attempted knapsack solution with Weights
void
Solve_KnapSack(Weights_T Weights) {
	Weight_T Target;
	Weights_T	CopyWeights,
					Complement;
	KnapResult_T	Result;

	CopyWeights = Copy_Stack_R(Weights);
	cout	<< "\nInput Target ==> ";
	cin	>> Target;
	Result = Do_Knap_R(Target, CopyWeights);
	if (Result.Q_Success) {
		cout	<< "\nSuccess with weights\n";
		Print_Stack_R(Result.Weights);
		cout	<< "\nleaving weights\n";
		Complement = Complement_R(Weights, Result.Weights);
		Print_Stack_R(Complement);
		Make_Empty_R(Result.Weights);
		Free_Stack(Result.Weights);
		Make_Empty_R(Complement);
		Free_Stack(Complement);
	}
	else {
		cout	<< "\nCannot make up "
				<< Target
				<< " from \n";
		Print_Stack_R(Weights);
	}
	Make_Empty_R(CopyWeights);
	Free_Stack(CopyWeights);
}




// PRE  -	TRUE
// POST -	menu displayed
void
Menu() {
	cout	<<	"***	(s)et up weights      ****************\n"
			<<	"***	solve (k)napsack      ****************\n"
			<< "***  (m)enu						  ************\n"
			<< "***  e(x)it						  ************\n";
}


// PRE	TRUE
// POST	Initialisation is done
void
Initialise(Weights_T &Weights) {

	Weights = Create_Stack();
	cout	<< "Here we go !\n";
}

void
test() {
	int i, j, Top;
	Stack_T stack;

	cout << "Type top";
	cin >> Top;

	stack = Create_Stack();
	for (i = 0; i <100; i++) {
		for (j = 0; j < Top; j++) {
			Push(j, stack);
		}
		for (j = 0; j < Top; j++) {
			Pop(stack);
		}
		StackStats();
	}
}

// PRE	TRUE
// POST Goes into a prompt, read and execute cycle
//		until 'x' is the character read
void
Loop(Weights_T &Weights) {

	char	Request;
	int	RequestCount;

	for (Request = 'm', RequestCount = 0; Request != 'x'; RequestCount++) {

		switch(Request) {

			case 's' :	Set_Up_Weights(Weights);
				break;
			case 'k' :	Solve_KnapSack(Weights);
				break;
			case 'c' :	StackStats();
				break;
			case 't' :	test();
				break;
			case 'm' : Menu();
				break;
			default   :
				cout	<< "\n m for menu \n";
		}
		cout	<< "\n\nNext request ["
				<< RequestCount
				<< "] ==> ";
		cin	>> Request;
	}
}


// PRE	TRUE
// POST	Clears up
void
Finalise() {
	cout	<< "That is it !\n";
}


void
main() {
	Weights_T	Weights;

	Initialise(Weights);
	Loop(Weights);
	Finalise();
}