// Ken Brownsey
// 31/8/98
// bst_tree.cpp
// BST_T directly pointer based implementation

#include "my_const.h"
#include "bst_tree.h"

// node structure for linked tree
typedef struct TN_T {
	struct TN_T	*Left_Ptr,
					*Right_Ptr;
	Element_T	Element;
} Node_T, *Node_Ptr_T;


int	NumberAllocated = 0;


// PRE	TRUE
// POST	RETURNS the empty tree
BST_T
Create_Tree() {
	return(NULL);
}


// PRE	TRUE
// POST	IF Tree is empty RETURNS TRUE ELSE RETURNS FALSE
BOOLEAN
Q_EmptyTree(BST_T Tree) {
	return(Tree == NULL);
}


// PRE  	Tree is not empty
// POST	RETURNS the root record
Element_T
Get_Root(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	return(Node_Ptr->Element);
}


// PRE	TRUE
// POST	IF Tree is not empty
//				RETURNS the left sub tree of Tree
//			ELSE
//				RETURNS the empty tree
BST_T
Get_Left(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	if (Node_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((void *) Node_Ptr->Left_Ptr);
	}
}


// PRE	TRUE
// POST	IF Tree is not empty
//				RETURNS the right sub tree of Tree
//			ELSE
//				RETURNS the empty tree
BST_T
Get_Right(BST_T Tree) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) Tree;

	if (Node_Ptr == NULL) {
		return(NULL);
	}
	else {
		return((void *) Node_Ptr->Right_Ptr);
	}
}


// PRE	TRUE
// POST	RETURNS the tree whose root is Root,
//				left sub tree is Left,
//				right sub tree is Right
BST_T
Make_Tree(BST_T Left, Element_T Element, BST_T Right) {

	Node_Ptr_T	NewTree;

	NewTree = new Node_T;
	NumberAllocated++;
	NewTree->Element = Element;
	NewTree->Left_Ptr = (Node_Ptr_T) Left;
	NewTree->Right_Ptr = (Node_Ptr_T) Right;
	return((void *) NewTree);
}


// PRE  - 	TRUE
// POST - 	RETURNS the number of nodes allocated
Get_NumberNodes_Allocated() {
	return(NumberAllocated);
}


// PRE	TRUE
// POST	if record with key Key is in BST
//				returnS TRUE
//			ELSE
//				returnS FALSE
BOOLEAN
Q_BST_Present_R(Key_T Key, BST_T BST) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) BST;

	if (BST == NULL) {
		return(FALSE);
	}
	else {
		if (Key < Get_Key(Node_Ptr->Element)) {
			return(Q_BST_Present_R(Key, Node_Ptr->Left_Ptr));
		}
		else {
			if (Key > Get_Key(Get_Root(BST))) {
				return(Q_BST_Present_R(Key, Node_Ptr->Right_Ptr));
			}
			else {
				return(TRUE);
			}
		}
	}
}


// PRE	NOT Q_BST_Present_R(Rec, BST)
// POST	BST' = BST with Rec inserted
void
BST_Insert_R(Element_T Rec, BST_T &BST) {
	Node_Ptr_T	Node_Ptr;
	Node_Ptr_T	NewNode_Ptr;

	Node_Ptr = (Node_Ptr_T) BST;
	if (Node_Ptr == NULL) {
		NewNode_Ptr = new Node_T;
		NumberAllocated++;
		NewNode_Ptr->Element = Rec;
		NewNode_Ptr->Left_Ptr = NULL;
		NewNode_Ptr->Right_Ptr = NULL;
		BST = NewNode_Ptr;
	}
	else {
		if (Get_Key(Rec) < Get_Key(Node_Ptr->Element)) {
			BST_Insert_R(Rec, (void *) (Node_Ptr->Left_Ptr));
		}
		else {
			BST_Insert_R(Rec, (void *) Node_Ptr->Right_Ptr);
		}
	}
}


// PRE	Q_BST_Present_R(Key, BST)
// POST	returnS record with key value Key from BST
Element_T
BST_Retrieve_R(Key_T Key, BST_T BST) {
	Node_Ptr_T	Node_Ptr;

	Node_Ptr = (Node_Ptr_T) BST;

	if (Key < Get_Key(Node_Ptr->Element)) {
		return(BST_Retrieve_R(Key, Node_Ptr->Left_Ptr));
	}
	else {
		if (Key > Get_Key(Get_Root(BST))) {
			return(BST_Retrieve_R(Key, Node_Ptr->Right_Ptr));
		}
		else {
			return(Node_Ptr->Element);
		}
	}
}


// PRE	NOT Q_EmptyTree(BST)
// POST	returnS rightmost Element_T of BST
Element_T
Get_RightMost_R(	Node_Ptr_T	Node_Ptr) {

	if (Node_Ptr->Right_Ptr == NULL) {
		return(Node_Ptr->Element);
	}
	else {
		return(Get_RightMost_R(Node_Ptr->Right_Ptr));
	}
}


// PRE	TRUE
// POST	if (Q_BST_Present(Rec, BST)
//				returnS BST with Rec deleted
//			ELSE
//				returnS BST
void
BST_Delete_R(Key_T Key, BST_T &BST) {
	Node_Ptr_T	Node_Ptr;
	Element_T NewRoot;

	Node_Ptr = (Node_Ptr_T) BST;

	if (Node_Ptr != NULL) {

		if (Key < Get_Key(Node_Ptr->Element)) {
			BST_Delete_R(Key, (void *) Node_Ptr->Left_Ptr);
		}

		else {
			if (Key > Get_Key(Node_Ptr->Element)) {
				BST_Delete_R(Key, (void *) Node_Ptr->Right_Ptr);
			}
			else {
				if (Node_Ptr->Left_Ptr == NULL) {
					BST = (void *) Node_Ptr->Right_Ptr;
					NumberAllocated--;
					delete(Node_Ptr);
				}
				else {
					if (Node_Ptr->Right_Ptr == NULL) {
						BST = (void *) Node_Ptr->Left_Ptr;
						NumberAllocated--;
						delete(Node_Ptr);
					}
					else {
						NewRoot = Get_RightMost_R(Node_Ptr->Left_Ptr);
						Node_Ptr->Element = NewRoot;
						BST_Delete_R(Get_Key(NewRoot), (void *) Node_Ptr->Left_Ptr);
					}
				}
			}
		}
	}
}


